//
//  Photo.swift
//  MarsRover
//
//  Created by Amit Bijlani on 10/20/16.
//  Copyright © 2016 Treehouse. All rights reserved.
//

import Foundation

let API_KEY = "PMBJGI8eXpgO4cyjYhRChf2mwt8YdP4xeT03oj3C"
let BASE_URL = "https://api.nasa.gov/mars-photos/api/v1/rovers/curiosity/photos"

typealias JSONDictionary = [String: Any]

enum SerializationError: Error {
    case missing(String)
    case invalid(String, Any)
}

struct Photo {
    public let imageURL: String
    public init(imageURL: String) {
        self.imageURL = imageURL
    }
}

extension Photo {
    init(dictionary: JSONDictionary) throws {
        guard let imageSRC = dictionary["img_src"] as? String else {
            throw SerializationError.missing("img_src")
        }
        
        self.imageURL = imageSRC.replacingOccurrences(of: "http", with: "https")
    }
}


extension Photo {
    static var urlComponents: URLComponents = {
        var u = URLComponents(string: BASE_URL)!
        u.queryItems =  [
            URLQueryItem(name: "sol", value: "1000"),
            URLQueryItem(name: "api_key", value: API_KEY)
        ]
        return u
    }()
    
    static func loadPhotosSync() -> [Photo] {
        var photos = [Photo]()
        
        if  let data = try? Data(contentsOf: urlComponents.url!),
            let json = try? JSONSerialization.jsonObject(with: data, options: []) as? JSONDictionary {
            //            print(json)
            for dictionary in (json?["photos"] as?  [JSONDictionary])!  {
                
                if let photo = try? Photo(dictionary: dictionary) {
                    photos.append(photo)
                }
            }
        }
        
        return photos
    }
    
    static func loadPhotosAsync(completion: @escaping ([Photo]) -> Void) {
        URLSession.shared.dataTask(with: urlComponents.url!) { (data, response, error) in
            var photos = [Photo]()
            
            if  let data = data,
                let json = try? JSONSerialization.jsonObject(with: data, options: []) as? JSONDictionary {
                //            print(json)
                for dictionary in (json?["photos"] as?  [JSONDictionary])!  {
                    
                    if let photo = try? Photo(dictionary: dictionary) {
                        photos.append(photo)
                    }
                }
            }
            completion(photos)
        }.resume()
    }

    
}

















